/* basics.cc
 *
 * Copyright (C) 1997 Technical University of Chemnitz
 * Oliver Langer
 *
 * This file contains the implementation of the class c_basics, where
 * the basic behaviours of all clients that connect to the server
 * (player/coach) are implemented. These behaviours include reveiving sensor 
 * information (visual and auditory information).
 */


static char implent[] = "$Id: basics.cc,v 0.9 1998/05/13 10:56:20 ola Stable $";


#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <iostream.h>
#include "basics.h"
#include "kernel.h"
#include "soccer_env.y.h"


/* BUGFIX_0
 * sometimes the message "flag p [l|r] [t|b]" is given with VDirection (=-0)
 * and VDistance (=0) ?!?! i haven't find error up to now -> maybe in server ?
 * fixed with the following macro so far...
 */                             
#define BUGFIX_0 \
	obj_data.dir_change = obj_data.dist_change = -1



const char *soccer_play_modes [] = {
	"before_kick_off",
	"time_over",
	"play_on",
	"kick_off",
	"kick_in",
	"free_kick",
	"corner_kick",
	"goal_kick",
	"goal",
	"throw_in"
};


c_basics::c_basics( void ) {
	server_link = NULL;

	SP_init_soccer_parser( NULL, msg, MAX_MSG_LEN );
	SP_set_scu_receiver( this );
}


c_basics::c_basics( char *teamname ) {
	server_link = NULL;

	SP_init_soccer_parser( teamname, msg, MAX_MSG_LEN );
	SP_set_scu_receiver( this );
}


c_basics::~c_basics() {
	SP_done_soccer_parser();
}


void c_basics::set_connection( c_connection *connection ) {
	server_link = connection;
}


void c_basics::release_connection( void ) {
	server_link = NULL;
}


void c_basics::leave_msg_loop( void ) {
	if( kernel_ptr != NULL )
		kernel_ptr->exit_kernel();
}


void c_basics::say( const char *format, ... ) {
	static char work[ MAX_MSG_LEN ];
	va_list args;

	va_start( args, format );

	vsprintf( work, format, args );
	sprintf( msg, "(say %s)\n", work );

	SEND( msg );

	va_end( args );
}


void c_basics::check_for_sensor( void ) {
	char sensor_type[ 5 ], s[ MAX_MSG_LEN ];
	float simtime;

	if( !RECV( msg, MAX_MSG_LEN ) )
		// no messages existing
		return;

	SP_parse_input_buffer();
}


void c_basics::hear_from_referee( float simtime, char *msg ) {
	char side;

	if( !strcmp( msg, "before_kick_off" ) ) {
		mode_changed( before_kick_off );
		return;
	}

	if( !strcmp( msg, "time_over" ) ) {
		mode_changed( time_over );
		return;
	}

	if( !strcmp( msg, "play_on" ) ) {
		mode_changed( play_on );
		return;
	}

	if( !strncmp( msg, "kick_off", 8 ) ) {
		sscanf( msg, "kick_off_%c", &side );
		mode_changed( kick_off, side );
		return;
	}

	if( !strncmp( msg, "kick_in", 7 ) ) {
		sscanf( msg, "kick_in_%c", &side );
		mode_changed( kick_in, side );
		return;
	}

	if( !strncmp( msg, "free_kick", 9 ) ) {
		sscanf( msg, "free_kick_%c", &side );
		mode_changed( free_kick, side );
		return;
	}

	if( !strncmp( msg, "corner_kick", 11 ) ) {
		sscanf( msg, "corner_kick_%c", &side );
		mode_changed( corner_kick, side );
		return;
	}

	if( !strncmp( msg, "goal_kick", 9 ) ) {
		sscanf( msg, "goal_kick_%c", &side );
		mode_changed( goal_kick, side );
		return;
	}

	if( !strncmp( msg, "goal", 4 ) ) {
		sscanf( msg, "goal_%c", &side );
		mode_changed( goal, side );
		return;
	}
}


/* set_kernel_ptr
 * For internal use only! It sets kernel_ptr to the active
 * c_kernel-instance. It is called by c_kernel::set_object.
 */
void c_basics::set_kernel_ptr( c_kernel *new_kernel_ptr ) {
	kernel_ptr = new_kernel_ptr;
}


void c_basics::wait_for_server( void ) {
	usleep( 500 );
}
