#ifndef __BASICS_H
#define __BASICS_H

#include "default.h"
#include "connection.h"
#include "libsclient.h"


#define SEND(s) server_link->send_msg( (s), strlen( (s) ) )
#define RECV(s,m) server_link->recv_msg( (s), (m) )


enum play_mode_t {
	before_kick_off = 0, 
	time_over = 1,
	play_on = 2,
	kick_off = 3,
	kick_in = 4,
	free_kick = 5,
	corner_kick = 6,
	goal_kick = 7,
	goal = 8,
	throw_in = 9
};


extern const char *soccer_play_modes[];

enum obj_info_contents_t {
	HIGH_DETAIL = 0, 	/* player sees everything */
	MEDIUM_DETAIL,		/* player does not see deltas */
	LOW_DETAIL		/* player sess direction only */
	
};

struct obj_info_t {
	float                   	distance, 
					direction,  
					dist_change,
					dir_change,
					face_dir;
	enum obj_info_contents_t	see_detail;
}; 


class c_kernel;

class c_basics {
	public:
		c_basics( void );
		c_basics( char * );
		~c_basics();

		virtual void set_connection( c_connection * );
		virtual void release_connection( void );

		virtual void leave_msg_loop( void );

		virtual void say( const char *, ... );

		virtual void hear_from_referee( float, char * );
		virtual void hear_from_direction( float, float, char *) = 0;

		virtual void mode_changed( enum play_mode_t mode, 
		  char side = 0 ) = 0;

		virtual void see_flag( float, struct obj_info_t, FlagPos ) = 0;
		virtual void see_flag_penalty( float, struct obj_info_t, FlagPos ) = 0;
		virtual void see_flag_center( float, struct obj_info_t ) = 0;
		virtual void see_flag_goal( float, struct obj_info_t, FlagPos ) = 0;
		virtual void see_flag_outside( float, struct obj_info_t, FlagPos, int ) = 0;
		virtual void see_flag_outside( float, struct obj_info_t, FlagPos ) = 0;
		virtual void see_flag( float, struct obj_info_t ) = 0;		

		virtual void see_goal( float, struct obj_info_t, Side ) = 0;
		virtual void see_goal( float, struct obj_info_t ) = 0;
		virtual void see_line( float, struct obj_info_t, LinePos ) = 0;
		virtual void see_line( float, struct obj_info_t ) = 0;
		virtual void see_ball( float, struct obj_info_t ) = 0;

		virtual void see_player( float, struct obj_info_t, WhichTeam, int ) = 0;
		virtual void see_player( float, struct obj_info_t, WhichTeam ) = 0;
		virtual void see_player( float, struct obj_info_t ) = 0;

		virtual void see_player_and_face( float, struct obj_info_t, WhichTeam, int, float ) = 0; /* NEW - Protocol 4 */
		virtual void see_player_and_face( float, struct obj_info_t, WhichTeam, float ) = 0; /* NEW - Protocol 4 */
		virtual void see_player_and_face( float, struct obj_info_t, float ) = 0; /* NEW - Protocol 4 */

		virtual void check_for_sensor( void );

		virtual void idle( void ) = 0;
		virtual void bye( void ) = 0;

		void wait_for_server( void );

		void set_kernel_ptr( c_kernel *new_kernel_ptr );

	protected: 
		c_connection		*server_link;
		char			msg[ MAX_MSG_LEN ];
		c_kernel		*kernel_ptr;
};

#endif
