/* connection.cc
 *
 * Copyright (C) 1997 Technical University of Chemnitz
 * Oliver Langer
 *
 * This file contains simple methods of the class c_connection by which a 
 * tcp/ip-connection can basically be managed (open/close/read/write)
 */


static char implent[] = "$Id: connection.cc,v 0.9 1998/05/13 10:56:20 ola Stable $";


#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include "connection.h"
#include "default.h"


c_connection::c_connection( void ) {
	server_host = NULL;
	portnr = DEFAULT_SERVER_PORT;
}


c_connection::~c_connection() {
	this->shutdown();
}


int c_connection::open( const char *_server_host, int _portnr ) {
	struct hostent *host_ref;
	struct sockaddr_in my_addr;
	int cur_flag;
	char	forget_char;
	int	nr_forget;

	portnr = _portnr;

	if( !(host_ref = gethostbyname( _server_host )) ) {
		/* do we have a ip-nr ?*/
		if( inet_addr( _server_host ) == -1 )
			return -1;
	} else {
		if( server_host != NULL ) 
			this->shutdown();

		server_host = (char *) malloc( strlen( _server_host ) + 1 );
		if( server_host != NULL )
			strcpy( server_host, _server_host );

		_server_host = inet_ntoa( *((struct in_addr *) *host_ref->h_addr_list ) );
	}

	if( (socket_fd = socket( AF_INET, SOCK_DGRAM, 0 ) ) < 0 )
		return -1;

	cur_flag = fcntl( socket_fd, F_GETFL, 0 );
	fcntl( socket_fd, F_SETFL, cur_flag | O_NONBLOCK );

	bzero((void *) &my_addr, sizeof(my_addr)) ;
	my_addr.sin_family = AF_INET ;
	my_addr.sin_addr.s_addr = htonl(INADDR_ANY) ;
	my_addr.sin_port = htons(0) ;

	if(bind(socket_fd, (struct sockaddr *) &my_addr, sizeof(my_addr)) < 0)
		return -1;

	bzero( (char *) &socket_addr, sizeof(socket_addr) );
    	socket_addr.sin_family = AF_INET ;
    	socket_addr.sin_addr.s_addr = inet_addr(_server_host) ;
    	socket_addr.sin_port = htons(portnr) ;

	/* read from socket, until there are lying no more characters */
	do {
		nr_forget = read( socket_fd, &forget_char, 1 );
	} while( nr_forget > 0 );

	return 0 ;
}


void c_connection::shutdown( void ) {
	if( server_host != NULL ) {
		close( socket_fd );

		free( server_host );
		server_host = NULL;
	}
}


int c_connection::send_msg( char *msg, unsigned int len ) {
	if( sendto( socket_fd, msg, len, 0, (struct sockaddr *) &socket_addr,
	  sizeof( socket_addr ) ) )
		return -1;

	return 0;
}


int c_connection::recv_msg( char *msg, unsigned int len ) {
	int bytes_read, addr_len;

	addr_len = sizeof( socket_addr );
	bytes_read = recvfrom( socket_fd, msg, len, 0, 
	  (struct sockaddr *) &socket_addr, &addr_len );

	if( bytes_read > 0 ) 
		msg[ bytes_read ] = 0;

	return( (bytes_read == -1) ? 0: bytes_read );
}
