/* kernel.cc
 *
 * Copyright (C) 1997 Technical University of Chemnitz
 * Oliver Langer
 *
 * This file contains the methods of the class c_kernel. This class just 
 * realizes a simple scheduling mechanism by which the two processes
 * (perfoming actions, receiving sensor data) are managed.
 */


static char implent[] = "$Id: kernel.cc,v 0.6 1998/04/30 07:11:27 ola Stable $";


#include <stddef.h>
#include <iostream.h>
#include "kernel.h"


c_kernel::c_kernel( void ) {
	run_state = 1;
	obj = NULL;
	incoming_freq = DEFAULT_INCOMING_FREQ;
}


c_kernel::~c_kernel() {
}


void c_kernel::set_object( c_basics *_obj ) {
	obj = _obj;
	if( obj != NULL ) 
		obj->set_kernel_ptr( this );
}


void c_kernel::set_incoming_freq( int new_freq ) {
	incoming_freq = new_freq;
}


void c_kernel::run( void ) {
	int no_check;

	if( obj == NULL ) {
		cerr << "c_kernel::run : no client installed !\n";
		return;
	}

	if( run_state == 2 )
		run_without_idle();

	do {
		no_check = incoming_freq;

		while( run_state == 1 ) {
			no_check--;
			if( !no_check ) {
				obj->check_for_sensor();
				no_check = incoming_freq;
			}

			obj->idle();
		}

		if( run_state )
			run_without_idle();
	} while( run_state );

	obj->bye();
}


void c_kernel::exit_kernel( void ) {
	run_state = 0;
}


void c_kernel::enable_idle( void ) {
	run_state = 1;
}


void c_kernel::disable_idle( void ) {
	run_state = 2;
}


void c_kernel::run_without_idle( void ) {
	while( run_state == 2 )
		obj->check_for_sensor();
}
