/*
 *	Header:
 *	File: libsclient.h
 *	Author: Noda Itsuki
 *	Date: 1997/04/1
 *	EndHeader:
 */
#ifndef __LIBSCLIENT_H
#define __LIBSCLIENT_H

#define Bool		int
#define SEENOBJMAX	32
#define BUFSIZE		256
#define TRUE		1
#define FALSE		0
#define DIST_ERR	-360
#define DIR_ERR		-360
#define CHNG_ERR	1024
#define UNUM_ERR	-1

#define PITCH_LENGTH	105.0
#define PITCH_WIDTH	68.0
#define PENALTY_AREA_LENGTH	16.5
#define PENALTY_AREA_WIDTH	40.32

#define Abs(x)  (((x) > 0.0)?(x):(-(x)))
#define Pow(x)  ((x)*(x))
#define Rad2Deg(x)  ((double)(x) * 180.0 / M_PI)
#define Deg2Rad(x)  ((double)(x) * M_PI / 180.0)

#ifdef MEMSET
#define bzero(a, b) memset(a, 0, b)
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

typedef struct socket{
		int socketfd  ;
		struct sockaddr_in serv_addr ;
} Socket ;

typedef enum _Side {
		S_UNKNOWN,
        S_LEFT,
        S_RIGHT,
} Side ;

#define PLAYMODE_STRINGS {\
	"before_kick_off",\
	"play_on",\
	"time_over",\
	"kick_off_l",\
	"kick_off_r",\
	"kick_in_l",\
	"kick_in_r",\
	"free_kick_l",\
	"free_kick_r",\
	"corner_kick_l",\
	"corner_kick_r",\
	"goal_kick_l",\
	"goal_kick_r",\
	"goal_l",\
	"goal_r",\
	NULL\
}

typedef enum _PlayMode {
        PM_BeforeKickOff,
        PM_PlayOn,
        PM_TimeOver,
        PM_KickOff_L,
        PM_KickOff_R,
        PM_KickIn_L,
        PM_KickIn_R,
        PM_FreeKick_L,
        PM_FreeKick_R,
        PM_CornerKick_L,
        PM_CornerKick_R,
        PM_GoalKick_L,
        PM_GoalKick_R,
        PM_Goal_L,
        PM_Goal_R,
        PM_Error,
} PlayMode ;

typedef struct _InitInfo {
        Side side ;
        int unum ;
        char playmodestr[32] ;
        PlayMode playmode ;
} InitInfo ;


typedef enum _ViewWidth {
        VW_Narrow,
        VW_Normal,
        VW_Wide,
} ViewWidth ;

typedef enum _ViewQuality {
        VQ_high,
        VQ_low,
} ViewQuality ;

typedef enum _ObjectType {
		OT_Unknown,
        OT_Ball,
        OT_Player,
        OT_Goal,
        OT_Flag,
        OT_Line,
} ObjectType ;

typedef enum _FlagPos {
		FP_Unknown,
        FP_L_T,
        FP_L_B,
        FP_R_T,
        FP_R_B,
        FP_C_T,
        FP_C_B,
        FP_P_L_T,
        FP_P_L_C,
        FP_P_L_B,
        FP_P_R_T,
        FP_P_R_C,
        FP_P_R_B,

	/* New Flags: */
	FP_C,
	FP_G_L_T,
	FP_G_L_B,
	FP_G_R_T,
	FP_G_R_B,

	FP_O_T_L,
	FP_O_T_R,
	FP_O_B_L,
	FP_O_B_R,
	FP_O_L_T,
	FP_O_L_B,
	FP_O_R_T,
	FP_O_R_B,
	FP_O_T,
	FP_O_B,
	FP_O_L,
	FP_O_R
} FlagPos ;

typedef enum _LinePos {
		LP_Unknown,
        LP_L,
        LP_T,
        LP_B,
        LP_R, 
} LinePos ;

typedef enum _WhichTeam {
        WT_Opponent = -1,
        WT_Unknown = 0,
        WT_Our = 1,
} WhichTeam ;

typedef struct _Ppos {
		WhichTeam team ;
		int unum ;
} PlayerInfo ;

typedef struct _SeenObj {
        ObjectType type ;
        Bool inViewConeP ; /* True if the object is in the view cone */
        union {
				PlayerInfo player ;
                Side side ;
                FlagPos fpos ;
		int	flag_outside_dist;
                LinePos lpos ;
        } id ;
        double dist ;
        double dir ;
        double dist_chng ;
        double dir_chng ;
	double face_dir;
} SeenObj ;

typedef struct _SeeInfo {
        int time ;
        int n_obj ;
        SeenObj obj[SEENOBJMAX] ;
} SeeInfo ;

typedef enum _SenderType {
        ST_Self,
        ST_Referee,
        ST_Player,
	ST_Goalie
} SenderType ;

typedef struct _HearInfo {
        int time ;
        SenderType sender ;
/* TODO: The following two fields are only necessary for a coach, receiving
 * a message from a player. So there has to be a separate structure for this situation
 */
	int player_nr;
	char team[BUFSIZE];
/*---*/
        double direction ;
        char message[BUFSIZE] ;
} HearInfo ;

typedef enum _SensorInfoType {
        SIT_See,
        SIT_Hear,
} SensorInfoType ;

typedef struct _SensorInfo {
        SensorInfoType type ;
        union {
                SeeInfo see ;
                HearInfo hear ;
        } info ;
} SensorInfo ;

typedef struct _XYPos {
        double x ;
        double y ;
} Pos ;

typedef struct _PosState {
        Pos pos ;
        double dir ;
} PosState ;

#endif
