/* player.cc
 *
 * Copyright (C) 1997 Technical University of Chemnitz
 * Oliver Langer
 * 
 * This file contains the implementation of the class c_player. All possible
 * actions of a client may be perfomed by calling the corresponding
 * methods of this class. It also contains methods which are called if a client
 * receives visual or auditory information.
 */


static char implent[] = "$Id: player.cc,v 0.9 1998/05/13 10:56:20 ola Stable $";


#include <iostream.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "player.h"
#include "soccer_env.y.h"



struct init_info_t * c_player::init( const char *team_name ) {
	return this->init( team_name, ST_Player );
}


struct init_info_t * c_player::init( const char *team_name, SenderType sender_type ) {
	char s[ 40 ];

	if( server_link == NULL ) {
		cerr << "c_player::init : no connection specified!\n";
		return NULL;
	}

	SP_set_teamname( (char *) team_name );

	sprintf( msg, "(init %s (%s))\n", team_name, (sender_type == ST_Goalie) ? "goalie" : "version 4.00" );
	
	SEND( msg );

	wait_for_server();

	RECV( msg, MAX_MSG_LEN );

	sscanf( msg, "(init %c %d %[^)])", &init_info.play_side, 
	  &init_info.play_nr, s );

	if( !strcmp( s, "before_kick_off" ) )
		init_info.play_mode = before_kick_off;
	else	
	if( !strcmp( s, "kick_off" ) )
		init_info.play_mode = kick_off;
	else
	if( !strcmp( s, "play_on" ) )
		init_info.play_mode = play_on;
	else
	if( !strcmp( s, "throw_in" ) )
		init_info.play_mode = throw_in;
	else
	if( !strcmp( s, "corner_kick" ) )
		init_info.play_mode = corner_kick;
	else
	if( !strcmp( s, "goal_kick" ) )
		init_info.play_mode = goal_kick;
	else
		return NULL;

	strcpy( init_info.team_name, team_name );

	return &init_info;
	
}
void c_player::move( float x, float y ) {
	sprintf( msg, "(move %f %f)\n", x, y );

	SEND( msg );
}


void c_player::turn( int moment ) {
	if( (moment < -180) || (moment > 180) ) {
		cerr << "turn: moment " << moment << " is out of range!\n";
		return;
	}

	sprintf( msg, "(turn %d)\n", moment );
	
	SEND( msg );
}


void c_player::dash( int power ) {
	if( (power < -30) || (power > 100) ) {
		cerr << "dash: power " << power << " is out of range!\n";
		return;
	}

	sprintf( msg, "(dash %d)\n", power );
	
	SEND( msg );
}


void c_player::kick( int power, int direction ) {
	if( (power < -30) || (power > 100) ) {
		cerr << "kick: power " << power << " is out of range!\n";
		return;
	}

	if( (direction < -180) || (direction > 180) ) {
                cerr << "kick: direction " << direction << " is out of range!\n";
                return;
        }

	sprintf( msg, "(kick %d %d)\n", power, direction );

	SEND( msg );
}


void c_player::change_view( const char *width, const char *quality ) {
	if( strcmp( width, "wide" ) )
		if( strcmp( width, "normal" ) )
			if( strcmp( width, "narrow" ) ) {
				cerr << "change_view: unknown width (="
				  << width << ") given!\n";
				return;
			}

	if( strcmp( quality, "high" ) )
		if( strcmp( quality, "low" ) ) {
			cerr << "change_view: unknown quality (=" << quality
			  << ") given!\n";
			return;
		}
			
	sprintf( msg, "(change view %s %s\n", (int) width, quality );
	
	SEND( msg );
}

void c_player::reconnect( void ) {
	reconnect( init_info.team_name, init_info.play_nr );
}


void c_player::reconnect( char *team_name, int play_nr ) {
	sprintf( msg, "(reconnect %s %d)", team_name, play_nr );
	SEND( msg );
}



// ----------------------------------------------------------------- c_goalie
struct init_info_t * c_goalie::init( const char *team_name ) {
	return this->c_player::init( team_name, ST_Goalie );
}


void c_goalie::catch_ball( int dir ) {
	sprintf( msg, "(catch %d)", dir );
	SEND( msg );
}

