%{
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "libsclient.h"
#include "basics.h"
#include "coach.h"


#ifndef HEAR_MSG_LEN
#define HEAR_MSG_LEN	1024
#endif

enum seen_obj_t  { SEE_PLAYER, SEE_PLAYER_TEAM_NR, SEE_PLAYER_TEAM, SEE_BALL, SEE_GOAL, SEE_GOAL_SIDE,
  SEE_GOAL_WITH_CONE, SEE_FLAG, SEE_FLAG_P_X_Y, SEE_FLAG_X_Y, SEE_FLAG_WITHOUT_CONE, 
  SEE_FLAG_CENTER, SEE_FLAG_GOAL, SEE_FLAG_OUTSIDE_X_Y, SEE_FLAG_OUTSIDE, SEE_LINE_X_Y, SEE_LINE };

static SensorInfo		*cur_sensor_info = NULL;
static SeenObj			*cur_see_info = NULL;
static char			*own_teamname = NULL;
static char			*ptr_input_buffer = NULL;
static int			len_input_buffer = 0;
static c_basics			*scu_player = NULL;
static struct obj_info_t	obj_info;

typedef SeenObj *	seenobj_4_yacc;

extern char *yytext;
void yyerror( char * );

static char * read_audio_msg();
static void read_player_info();
static char * read_teamname();
static int read_playernr();

int yylex();

%}


%token SYMB_AV_HEAR SYMB_OBJ_REFEREE SYMB_OBJ_SELF SYMB_WORDSTRING
%token SYMB_MD_BEFORE_KICKOFF SYMB_MD_TIME_OVER SYMB_MD_PLAY_ON 
%token SYMB_MD_KICK_OFF_L SYMB_MD_KICK_OFF_R SYMB_MD_KICK_IN_L
%token SYMB_MD_KICK_IN_R SYMB_MD_FREE_KICK_L SYMB_MD_FREE_KICK_R 
%token SYMB_MD_CORNER_KICK_L SYMB_MD_CORNER_KICK_R SYMB_MD_GOAL_KICK_L
%token SYMB_MD_GOAL_KICK_R SYMB_MD_GOAL_L SYMB_MD_GOAL_R 
%token SYMB_WORDSTRING SYMB_INT_NR SYMB_FLOAT_NR SYMB_AV_SEE
%token SYMB_OBJ_PLAYER SYMB_OBJ_player SYMB_OBJ_BALL 
%token SYMB_OBJ_ball SYMB_OBJ_GOAL SYMB_OBJ_goal SYMB_OBJ_FLAG
%token SYMB_OBJ_flag SYMB_OBJ_line 
%token SYMB_OBJ_PENALTY SYMB_OBJ_GOALFLAG SYMB_OBJ_hv_pos SYMB_OBJ_hv_pos
%token R_BRAK L_BRAK


%type <int_nr>		sim_time, player_nr;
%type <str>		message, teamname;
%type <int_nr>		obj_name, flag_outside_dist;
%type <seen_obj>	notified_obj;
%type <float_nr>	expect_float;
%type <float_nr>	distance direction dist_change dir_change, face_dir;
%type <character> 	SYMB_OBJ_PENALTY SYMB_OBJ_hv_pos;



%union {
	float		float_nr;
	int		int_nr;
	seen_obj_t	seen_obj_type;
	SeenObj		*seen_obj;
	char		character;
	char		*str;
	Side		side;
}


%start incoming_msg

%%

incoming_msg: L_BRAK audio_visual R_BRAK
	;


audio_visual: audio_info
	| visual_info
	;

audio_info: SYMB_AV_HEAR sim_time SYMB_OBJ_REFEREE message 
		{
#ifdef __STORE_SENSOR_INFO
			cur_sensor_info->type = SIT_Hear;
			cur_sensor_info->info.hear.time = $2;
			cur_sensor_info->info.hear.sender = ST_Referee;

			/* Note: cur_sensor_info->info.hear.message will be filled by 
			 * the rule for <message>
			 */
#endif

#ifdef __USE_SCU
			scu_player->hear_from_referee( $2, $4 );
#endif
		}
	| SYMB_AV_HEAR sim_time L_BRAK SYMB_OBJ_player teamname player_nr R_BRAK message
		{
#ifdef __STORE_SENSOR_INFO
			cur_sensor_info->type = SIT_Hear;
			cur_sensor_info->info.hear.time = $2;
			cur_sensor_info->info.hear.sender = ST_Player;

			cur_sensor_info->info.hear.player_nr = $5;
			/* Note: cur_sensor_info->info.hear.team will be filled by the rule <teamname> */
#endif

#ifdef __USE_SCU
			/* The following cast is **very dangerous**, because we do not proof, wether the given
			 * c_basics-pointer points to an instance of c_coach or not. If not, it may (should) crash
			 * because the method used here is only defined in c_coach!
			 */
			((c_coach *) scu_player)->hear_from_player( $2, $5, $6, $8 );
#endif
		}
	| SYMB_AV_HEAR sim_time SYMB_OBJ_SELF message
		{ 
#ifdef __STORE_SENSOR_INFO
			cur_sensor_info->type = SIT_Hear;
			cur_sensor_info->info.hear.time = $2;
			cur_sensor_info->info.hear.sender = ST_Self;

			/* Note: cur_sensor_info->info.hear.message will be filled by 
			 * the rule for <message>
			 */
#endif

#ifdef __USE_SCU
			// not supported by SCU
#endif
		}
	| 
		SYMB_AV_HEAR sim_time direction message
		{

#ifdef __STORE_SENSOR_INFO
			cur_sensor_info->type = SIT_Hear;
			cur_sensor_info->info.hear.time = $2;
			cur_sensor_info->info.hear.sender = ST_Player;
			cur_sensor_info->info.hear.direction = $3;

			/* Note: cur_sensor_info->info.hear.message will be filled by 
			 * the rule for <message>
			 */
#endif

#ifdef __USE_SCU
			scu_player->hear_from_direction( $2, $3, $4 );
#endif
		}
	;


/* message automatically stores the message at the right place */
message: 
	{	
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		$$ = read_audio_msg();
#else
	WRITE YOUR OWN CODE FOR READIN IN THE MESSAGE HERE.
	YOU HAVE SIMPLY HAVE TO READ UNTIL ALL OPENING BRACKETS ARE CLOSED
#endif
	}
	;

	
sim_time: SYMB_INT_NR
	{
		$$ = yyval.int_nr;
	}
	;


direction: expect_float
	{
		$$ = $1;
	}
	;


expect_float: SYMB_FLOAT_NR
		{
			$$ = yyval.float_nr;
		}
	| SYMB_INT_NR 
		{
			$$ = (float) yyval.int_nr;
		}
	;


visual_info: SYMB_AV_SEE sim_time
	{
		cur_see_info = cur_sensor_info->info.see.obj;

		cur_sensor_info->type = SIT_See;
		cur_sensor_info->info.see.time = $2;
#ifdef __STORE_SENSOR_INFO
		cur_sensor_info->info.see.n_obj = 0;
#endif
	} notified_objs;


notified_objs: notified_obj 
	{	cur_see_info++;
		cur_sensor_info->info.see.n_obj++;
	} notified_objs
	| notified_obj
	{
		cur_sensor_info->info.see.n_obj++;
	}
	;


notified_obj: 
	L_BRAK L_BRAK obj_name R_BRAK distance direction dist_change dir_change face_dir R_BRAK 
	{	/* Note: <face_dir> is only given if a player sees another player. So we do not have
		 * to check for anything else.
		 */
#ifdef __STORE_SENSOR_INFO
		cur_see_info->dist = $5;
		cur_see_info->dir = $6;
		cur_see_info->dist_chng = $7;
		cur_see_info->dir_chng = $8;
		cur_see_info->face_dir = $9;
#endif

#ifdef __USE_SCU
		obj_info.distance = $5;
		obj_info.direction = $6;
		obj_info.dist_change = $7;
		obj_info.dir_change = $8;
		obj_info.face_dir = $9;
		obj_info.see_detail = HIGH_DETAIL;

		switch( $3 ) {
			case SEE_PLAYER:
				scu_player->see_player_and_face( cur_sensor_info->info.see.time, obj_info, $9 );
				break;
			case SEE_PLAYER_TEAM_NR:
				scu_player->see_player_and_face( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team, cur_see_info->id.player.unum, $9 );
				break;
			case SEE_PLAYER_TEAM:
				scu_player->see_player_and_face( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team, $9 );
				break;
		}
#endif
	}
	| L_BRAK L_BRAK obj_name R_BRAK distance direction dist_change dir_change R_BRAK 
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->dist = $5;
		cur_see_info->dir = $6;
		cur_see_info->dist_chng = $7;
		cur_see_info->dir_chng = $8;
#endif

#ifdef __USE_SCU
		obj_info.distance = $5;
		obj_info.direction = $6;
		obj_info.dist_change = $7;
		obj_info.dir_change = $8;
		obj_info.see_detail = HIGH_DETAIL;

		switch( $3 ) {
			case SEE_PLAYER:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_PLAYER_TEAM_NR:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team, cur_see_info->id.player.unum );
				break;
			case SEE_PLAYER_TEAM:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team );
				break;
			case SEE_BALL:
				scu_player->see_ball( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_GOAL:
			case SEE_GOAL_WITH_CONE:
				scu_player->see_goal( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_GOAL_SIDE:
				scu_player->see_goal( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.side );
				break;
			case SEE_FLAG_P_X_Y:
				scu_player->see_flag_penalty( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_CENTER:
				scu_player->see_flag_center( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_FLAG_GOAL:
				scu_player->see_flag_goal( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_OUTSIDE_X_Y:
				scu_player->see_flag_outside( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos,
				  cur_see_info->id.flag_outside_dist );
				break;
			case SEE_FLAG_OUTSIDE:
				scu_player->see_flag_outside( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_X_Y:
				scu_player->see_flag( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_LINE:
				scu_player->see_line( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_LINE_X_Y:
				scu_player->see_line( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.lpos );
				break;
		}
#endif
	}
	| L_BRAK L_BRAK obj_name R_BRAK distance direction R_BRAK
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->dist = $5;
		cur_see_info->dir = $6;
		cur_see_info->dist_chng = CHNG_ERR;
		cur_see_info->dist_chng = CHNG_ERR;
#endif
		
#ifdef __USE_SCU
		obj_info.distance = $5;
		obj_info.direction = $6;
		obj_info.see_detail = MEDIUM_DETAIL;

		switch( $3 ) {
			case SEE_PLAYER:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_PLAYER_TEAM_NR:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team, cur_see_info->id.player.unum );
				break;
			case SEE_PLAYER_TEAM:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team );
				break;
			case SEE_BALL:
				scu_player->see_ball( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_GOAL:
			case SEE_GOAL_WITH_CONE:
				scu_player->see_goal( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_GOAL_SIDE:
				scu_player->see_goal( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.side );
				break;
			case SEE_FLAG_P_X_Y:
				scu_player->see_flag_penalty( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_CENTER:
				scu_player->see_flag_center( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_FLAG_GOAL:
				scu_player->see_flag_goal( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_OUTSIDE_X_Y:
				scu_player->see_flag_outside( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos,
				  cur_see_info->id.flag_outside_dist );
				break;
			case SEE_FLAG_OUTSIDE:
				scu_player->see_flag_outside( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_X_Y:
				scu_player->see_flag( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_LINE:
				scu_player->see_line( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_LINE_X_Y:
				scu_player->see_line( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.lpos );
				break;
		}
#endif
	}
	| L_BRAK L_BRAK obj_name R_BRAK direction R_BRAK
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->dir = $5;
		cur_see_info->dist = DIST_ERR;
		cur_see_info->dist_chng = CHNG_ERR;
		cur_see_info->dist_chng = CHNG_ERR;
#endif
		
#ifdef __USE_SCU
		obj_info.distance = $5;
		obj_info.see_detail = LOW_DETAIL;

		switch( $3 ) {
			case SEE_PLAYER:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_PLAYER_TEAM_NR:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team, cur_see_info->id.player.unum );
				break;
			case SEE_PLAYER_TEAM:
				scu_player->see_player( cur_sensor_info->info.see.time, obj_info,
				  cur_see_info->id.player.team );
				break;
			case SEE_BALL:
				scu_player->see_ball( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_GOAL:
			case SEE_GOAL_WITH_CONE:
				scu_player->see_goal( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_GOAL_SIDE:
				scu_player->see_goal( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.side );
				break;
			case SEE_FLAG_P_X_Y:
				scu_player->see_flag_penalty( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_CENTER:
				scu_player->see_flag_center( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_FLAG_GOAL:
				scu_player->see_flag_goal( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_OUTSIDE_X_Y:
				scu_player->see_flag_outside( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos,
				  cur_see_info->id.flag_outside_dist );
				break;
			case SEE_FLAG_OUTSIDE:
				scu_player->see_flag_outside( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_FLAG_X_Y:
				scu_player->see_flag( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.fpos );
				break;
			case SEE_LINE:
				scu_player->see_line( cur_sensor_info->info.see.time, obj_info );
				break;
			case SEE_LINE_X_Y:
				scu_player->see_line( cur_sensor_info->info.see.time, obj_info, cur_see_info->id.lpos );
				break;
		}
#endif
	}
	;


obj_name: SYMB_OBJ_PLAYER
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Player;
		cur_see_info->inViewConeP = FALSE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.player.team = WT_Unknown;
		cur_see_info->id.player.unum = UNUM_ERR;
#endif
		
		$$ = SEE_PLAYER;
	}
	| SYMB_OBJ_player player_info
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Player;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		if(  cur_see_info->id.player.team != WT_Unknown )
			if(  cur_see_info->id.player.unum != UNUM_ERR )
				$$ = SEE_PLAYER_TEAM_NR;
			else 	$$ = SEE_PLAYER_TEAM;
		else	$$ = SEE_PLAYER;
#endif
	}
	| SYMB_OBJ_BALL
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Ball;
		cur_see_info->inViewConeP = FALSE;
#endif

		$$ = SEE_BALL;
	}
	| SYMB_OBJ_ball
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Ball;
		cur_see_info->inViewConeP = TRUE;
#endif

		$$ = SEE_BALL;
	}
	| SYMB_OBJ_GOAL
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Goal;
		cur_see_info->inViewConeP = FALSE;
		cur_see_info->id.side = S_UNKNOWN;
#endif
		$$ = SEE_GOAL;
	}
	| SYMB_OBJ_goal SYMB_OBJ_hv_pos
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Goal;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.side = ( $2 == 'l' ) ? S_LEFT : S_RIGHT;
#endif
		
		$$ = SEE_GOAL_SIDE;
	}
	| SYMB_OBJ_goal
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Goal;
		cur_see_info->inViewConeP = TRUE;
		cur_see_info->id.side = S_UNKNOWN;
#endif
		
		$$ = SEE_GOAL_WITH_CONE;
	}
	| SYMB_OBJ_flag SYMB_OBJ_PENALTY SYMB_OBJ_hv_pos SYMB_OBJ_hv_pos
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.fpos = FP_Unknown;

		switch( $4) {
			case 't': 
				cur_see_info->id.fpos = ( $3 == 'l' ) ?
				  FP_P_L_T : FP_P_R_T;
				break;
			case 'c':
				cur_see_info->id.fpos = ( $3 == 'l' ) ?
				  FP_P_L_C : FP_P_R_C;
				break;
			case 'b':
				cur_see_info->id.fpos = ( $3 == 'l' ) ?
				  FP_P_L_B : FP_P_R_B;
				break;
			//default: ??
		}
#endif
		$$ = SEE_FLAG_P_X_Y;
	}
	| SYMB_OBJ_flag SYMB_OBJ_GOALFLAG SYMB_OBJ_hv_pos SYMB_OBJ_hv_pos
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.fpos = FP_Unknown;

		switch( $3) {
			case 'l': 
				cur_see_info->id.fpos = ( $4 == 't' ) ?
				  FP_G_L_T : FP_G_L_B;
				break;
			case 'r':
				cur_see_info->id.fpos = ( $4 == 't' ) ?
				  FP_G_R_T : FP_G_R_B;
				break;
		}
#endif
		$$ = SEE_FLAG_GOAL;
	}
	| SYMB_OBJ_flag SYMB_OBJ_hv_pos SYMB_OBJ_hv_pos flag_outside_dist
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.fpos = FP_Unknown;

		switch( $2 ) {
			case 't': 
				cur_see_info->id.fpos = ($3 == 'l') ? FP_O_T_L : FP_O_T_R;
				break;
			case 'b':
				cur_see_info->id.fpos = ($3 == 'l') ? FP_O_B_L : FP_O_B_R;
				break;
			case 'l':
				cur_see_info->id.fpos = ($3 == 't') ? FP_O_L_T : FP_O_L_B;
				break;
			case 'r':
				cur_see_info->id.fpos = ($3 == 't') ? FP_O_R_T : FP_O_R_B;
				break;
		}
		cur_see_info->id.flag_outside_dist = $4;

#endif
		$$ = SEE_FLAG_OUTSIDE_X_Y;
	}
	| SYMB_OBJ_flag SYMB_OBJ_hv_pos flag_outside_dist {
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.fpos = FP_Unknown;

		switch( $2 ) {
			case 't': 
				cur_see_info->id.fpos = FP_O_T;
				break;
			case 'b':
				cur_see_info->id.fpos = FP_O_B;
				break;
			case 'l':
				cur_see_info->id.fpos = FP_O_L;
				break;
			case 'r':
				cur_see_info->id.fpos = FP_O_R;
				break;
		}
		cur_see_info->id.flag_outside_dist = -1;
#endif
		$$ = SEE_FLAG_OUTSIDE;
	}
	| SYMB_OBJ_flag SYMB_OBJ_hv_pos SYMB_OBJ_hv_pos 
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.fpos = FP_Unknown;

		switch( $2 ) {
			case 'l': 
				cur_see_info->id.fpos = ( $3 == 't' ) ?
				  FP_L_T : FP_L_B;
				break;
			case 'r':
				cur_see_info->id.fpos = ( $3 == 't' ) ?
				  FP_R_T : FP_R_B;
				break;
			case 'c':
				cur_see_info->id.fpos = ( $3 == 't' ) ?
				  FP_C_T : FP_C_B;
				break;
			//default: ??
		}
#endif
		$$ = SEE_FLAG_X_Y;
	}
	| SYMB_OBJ_flag SYMB_OBJ_hv_pos {
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.fpos = ($2 == 'c') ? FP_C : FP_Unknown;
#endif
		$$ = SEE_FLAG_CENTER;
	}
	| SYMB_OBJ_FLAG
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = FALSE;
		cur_see_info->id.fpos = FP_Unknown;
#endif

		$$ = SEE_FLAG_WITHOUT_CONE;
	}
	| SYMB_OBJ_flag
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Flag;
		cur_see_info->inViewConeP = TRUE;
		cur_see_info->id.fpos = FP_Unknown;
#endif

		$$ = SEE_FLAG;
	}
	| SYMB_OBJ_line SYMB_OBJ_hv_pos
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Line;
		cur_see_info->inViewConeP = TRUE;
#endif
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		switch( $2 ) {
			case 'l': cur_see_info->id.lpos = LP_L; break;
			case 'r': cur_see_info->id.lpos = LP_R; break;
			case 't': cur_see_info->id.lpos = LP_T; break;
			case 'b': cur_see_info->id.lpos = LP_B; break;
		}
#endif

		$$ = SEE_LINE_X_Y;
	}
	| SYMB_OBJ_line
	{
#ifdef __STORE_SENSOR_INFO
		cur_see_info->type = OT_Line;
		cur_see_info->inViewConeP = TRUE;
		cur_see_info->id.lpos = LP_Unknown;
#endif

		$$ = SEE_LINE;
	}
	;


teamname: 
	{ $$ = read_teamname(); }
	;


player_nr: { $$ = read_playernr(); }
	;

player_info:
	{	
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		read_player_info();
#else

	WRITE YOUR OWN PARSING FOR PLAYER INFO HERE. The string to be evalueated here consists
	of <name> [nr]
#endif
	}


distance: expect_float
	{
		$$ = $1;
	}
	;


dist_change: expect_float
	{
		$$ = $1;
	}
	;


dir_change: expect_float
	{
		$$ = $1;
	}
	;

flag_outside_dist: SYMB_INT_NR
	{
		$$ = yyval.int_nr;
	}
	;


face_dir: expect_float
	{
		$$ = $1;
	}
	;
%%
#include <stdlib.h>
#include "lex.yy.c"

void SP_init_soccer_parser( char *teamname, char *input_buffer, int buffer_len ) {
	cur_sensor_info = (SensorInfo *) calloc( 1, sizeof( SensorInfo ) );

	if( cur_sensor_info == NULL ) {
		fprintf( stderr, "soccer_env: Unable to allocate internal structures!\n" );
		exit( 1 );
	}

	if( teamname == NULL ) {
		own_teamname = (char *) malloc( 1 );
		own_teamname[ 0 ] = 0;
	} else {
		own_teamname = (char *) malloc( strlen( teamname ) + 1 );
		if( own_teamname != NULL )
			strcpy( own_teamname, teamname );
	}

	if( own_teamname == NULL ) {
		fprintf( stderr, "soccer_env: Unable to duplicate string!\n" );
		exit( 1 );
	}

	ptr_input_buffer = input_buffer;
	len_input_buffer = buffer_len;
}


void SP_set_teamname( char *name_of_team ) {
	if( name_of_team == NULL )
		return;

	if( own_teamname != NULL )
		free( own_teamname );

	if( name_of_team != NULL ) {
		own_teamname = (char *) malloc( strlen( name_of_team ) + 1 );

		if( own_teamname != NULL )
			strcpy( own_teamname, name_of_team );
	}
}


#ifdef __USE_SCU
void SP_set_scu_receiver( c_basics *receiver ) {
	scu_player = receiver;
}
#endif


void SP_set_input_buffer( char *ptr, int len ) {
	ptr_input_buffer = (ptr-1);
	len_input_buffer = len;
}


void SP_set_receiving_player( c_basics *player_obj ) {
	scu_player = player_obj;
}


char * read_audio_msg() {
	char	*c;
	int	cnt_brackets = 1, indx = 0;
	
	c = cur_sensor_info->info.hear.message;

	*c = (char) yyinput();	// our blank
	*c = (char) yyinput();
	while( (*c != 0) && (cnt_brackets > 0) ) {
		if( *c == ')' ) {
			cnt_brackets--;
			if( cnt_brackets == 0 ) {
				unput( *c );
				break;
			}
		} else	if( *c == '(' )
				cnt_brackets++;

		c++;
		*c = (char) yyinput();
	}

	*c = 0;

	return cur_sensor_info->info.hear.message;
}


char * read_teamname() {
	char	*c;
	int	cnt_brackets = 1, indx = 0;
	
	c = cur_sensor_info->info.hear.team;

	*c = (char) yyinput();	// our blank
	*c = (char) yyinput();
	while( *c != ' ' ) {
		*c = (char) yyinput();
	}

	unput( *c );

	*c = 0;

	return cur_sensor_info->info.hear.team;
}


int read_playernr() {
	char	c;
	int	cnt_brackets = 1, indx = 0, nr = 0;
	
	c = (char) yyinput();	// our blank
	c = (char) yyinput();

	while( (c != ' ' ) && (c != ')') ) {
		c = (char) yyinput();
		if( (c>='0') && (c<='9') )
			nr = nr * 10 + c - '0';
	}

	unput( c );

	return nr;
}


void read_player_info() {
	char	c;
	int	cnt_brackets = 1, i = 0, cur_nr;
	
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
	cur_see_info->id.player.team = WT_Unknown;
	cur_see_info->id.player.unum = UNUM_ERR;
#endif
	c = (char) yyinput();
	if( c != ')' ) {
		/* we have additional information about the player - so proceed */

#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		cur_see_info->id.player.team = WT_Our;
#endif

		c = (char) yyinput();
		while( (c != 0) && (cnt_brackets > 0) && (c != ' ') ) {
			if( c == ')' ) {
				cnt_brackets--;
				if( cnt_brackets == 0 ) {
					unput( c );
					break;
				}
			} else	if( c == '(' )
					cnt_brackets++;

			if( c != own_teamname[ i ] ) {
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
				cur_see_info->id.player.team = WT_Opponent;
#endif

				c = (char) yyinput();		

				/* we repeat the loop here for optimizing reasons */
				while( (c != 0) && (cnt_brackets > 0) && (c != ' ') ) {
					if( c == ')' ) {
						cnt_brackets--;
						if( cnt_brackets == 0 ) {
							unput( c );
							break;
						}
					} else	if( c == '(' )
							cnt_brackets++;

					c = (char) yyinput();		
				}

				break;
			}
			c = (char) yyinput();
			i++;
		}

		if( c == ' ' ) {
			/* player number is also available, which we have to read in now: */
			cur_nr = 0;
			do {
				c = (char) yyinput();

				if( c == ')' ) {
					cnt_brackets--;
					if( cnt_brackets == 0 ) {
						unput( c );
						break;
					}
				} else	if( c == '(' )
						cnt_brackets++;

				cur_nr = (cur_nr*10) + (int) c - 48;
			} while( cnt_brackets > 0 );

#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
			cur_see_info->id.player.unum = cur_nr;

#endif
		} 
#if defined( __USE_SCU ) || defined( __STORE_SENSOR_INFO )
		 else	cur_see_info->id.player.unum = UNUM_ERR;
#endif
	} else	unput( c );
}


void SP_parse_input_buffer( void ) {
	int	str_len;

	if( ptr_input_buffer == NULL ) {
		fprintf( stderr, "soccer_env: NULL pointer to input buffer!\n" );
		exit( 1 );
	}

#ifdef __USE_SCU
	if( scu_player == NULL ) {
		fprintf( stderr, "soccer_env: No player given!\n" );
		exit( 1 );
	}
#endif

#if defined(__PRINT_BUFFER)
	printf( "INPUT BUFFER >%s<\n", ptr_input_buffer );
#endif
	yy_scan_string( ptr_input_buffer );
	yyparse();
}


void SP_done_soccer_parser() {
	if( cur_sensor_info != NULL ) {
		free( cur_sensor_info );
		cur_sensor_info = NULL;
	}

	if( own_teamname != NULL ) {
		free( own_teamname );
		own_teamname = NULL;
	}
}


#ifdef __CHECK_GRAMMAR
#include <string.h>

#if defined( __USE_SCU )
#include "first.h"
#endif

void main( int argc, char *argv[] ) {
	char		*example_str;
	int		str_len;

	if( argc < 2 ) {
		fprintf( stderr, "No example string specified within command line!\n" );
		exit( 1 );
	}

	str_len = strlen( argv[ 1 ] );

	example_str = (char *) malloc( str_len + 2 );
	if( example_str == NULL ) {
		fprintf( stderr, "Failed to allocate string!" );
		exit( 1 );
	}
	strcpy( example_str, argv[ 1 ] );

#if defined( __USE_SCU )
	c_my_client		testclient;

	testclient.init( "MyTeam" );

	SP_set_scu_receiver( &testclient );

#endif
	SP_init_soccer_parser( "MyTeam", example_str, str_len );

	SP_parse_input_buffer();

	SP_done_soccer_parser();

	exit( 0 );
}
#endif


void yyerror( char *msg ) {
	fprintf( stderr, "ERROR: %s\n", msg );

	exit( 1 );

#if 0
#endif
}
